@@ -11,6 +11,7 @@ This moduleis similar in concept to click, but with fewer features.
 import sys
 import argparse
 import functools
+import datetime
 from typing import Any, Callable, Dict, Iterable, Mapping, Optional, Tuple, Union
 
 from ptutils.undefined import UNDEFINED, is_undefined, is_defined, default_if_undefined
@@ -145,14 +146,14 @@ def _arg_name(args: Tuple[Any], kwargs: Dict[str, Any]) -> str:
 
 
 # ------------------------------------------------------------------------------------------------------------------------
-def _resolve(name: Union[str, Callable]) -> str:
+def _resolve(subject: Union[str, Callable]) -> str:
     """
     Given a string or a function, determine the name the cli simple system will use for lookups
     for shared arguments and argument groups.
 
     Parameters
     ----------
-    name : Union[str, Callable]
+    subject : Union[str, Callable]
         A string or function.
 
     Returns
@@ -163,33 +164,28 @@ def _resolve(name: Union[str, Callable]) -> str:
     Raises
     ------
     TypeError
-        When `name` is not a string and not callable.
+        When `subject` is not a string and not callable.
     """
-    if callable(name):
-        f = name
-        name = f.__name__
-        if name.endswith('dummy'):
-            return f.__cmd_info__['arg_info'][0]
-        else:
-            return name
-    elif isinstance(name, str):
-        return name
+    if callable(subject):
+        return subject.__name__
+    elif isinstance(subject, str):
+        return subject
     else:
         raise TypeError(
-            f"Object of type '{type(name).__name__}' may not "
+            f"Object of type '{type(subject).__name__}' may not "
             "be used to reference a command, arg, arg_group, or shared_arg."
         )
 
 
 # ------------------------------------------------------------------------------------------------------------------------
-def _resolve_command(name: Union[str, Callable]) -> str:
+def _resolve_command(subject: Union[str, Callable]) -> str:
     """
     Given a string or a function, determine the name the cli simple system will use for lookups
     for commands.
 
     Parameters
     ----------
-    name : Union[str, Callable]
+    subject : Union[str, Callable]
         A string or function.
 
     Returns
@@ -200,9 +196,9 @@ def _resolve_command(name: Union[str, Callable]) -> str:
     Raises
     ------
     TypeError
-        When `name` is not a string and not callable.
+        When `subject` is not a string and not callable.
     """
-    return normalize_command_name(_resolve(name))
+    return normalize_command_name(_resolve(subject))
 
 
 # ------------------------------------------------------------------------------------------------------------------------
@@ -340,8 +336,10 @@ def remove_shared_arg(name: Union[str, Callable]) -> None:
     not also cleaned up.
     """
     name_ = _resolve(name)
-    if _resolve(name_) in __ARG_GROUPS__:
+    if name_ in __ARG_GROUPS__:
         del __ARG_GROUPS__[name_]
+    else:
+        raise Exception(f"No shared arg or arg group by name '{name_}', so nothing to delete.")
 
 
 # ------------------------------------------------------------------------------------------------------------------------
@@ -520,7 +518,7 @@ def _walk_argument_specs(
             if name not in __ARG_GROUPS__:
                 raise Exception(
                     f"Invalid argument group specification: Name '{name}' is "
-                    "not a registered argument group."
+                    f"not a registered argument group: {list(__ARG_GROUPS__.keys())}."
                 )
 
             child_subject     = __ARG_GROUPS__[name]
@@ -635,7 +633,7 @@ def arg(*args, **kwargs) -> Callable:
             "kind":   "arg",
             "args":   args,
             "kwargs": kwargs,
-            "name":   _arg_name(arg, kwargs)
+            "name":   _arg_name(args, kwargs)
         }
         func.__cmd_info__['arg_info'].append(spec)
         return func
@@ -643,7 +641,7 @@ def arg(*args, **kwargs) -> Callable:
 
 
 # ------------------------------------------------------------------------------------------------------------------------
-def use_arg_group(name: str) -> Callable:
+def use_arg_group(name: Union[str, Callable]) -> Callable:
     """
     Parametric decorator that adds a group of arguments to a command.
 
@@ -674,10 +672,14 @@ def use_arg_group(name: str) -> Callable:
         Callable
             The function with argument group metadata injected.
         """
+        name_ = _resolve(name)
+        if name_ not in __ARG_GROUPS__:
+            raise Exception(f"Unknown argument group or shared argument: '{name}'")
         spec = {
             "kind":   "arg_group",
-            "name":   name
+            "name":   name_
         }
+
         func.__cmd_info__['arg_info'].append( spec )
         return func
     return decorate
@@ -744,11 +746,11 @@ def arg_group(func):
     name = func.__name__
 
     @functools.wraps(func)
-    def dummy(**kwargs):
+    def __cli_internal__(**kwargs):
         raise Exception("An arg group is not callable")
 
-    _register_arg_group(name, dummy)
-    return dummy
+    _register_arg_group(name, __cli_internal__)
+    return __cli_internal__
 
 
 # ------------------------------------------------------------------------------------------------------------------------
@@ -775,20 +777,20 @@ def shared_arg(*args, **kwargs) -> None:
 
     name = _arg_name(args, kwargs)
 
-    def dummy():
+    def __cli_internal__():
         raise Exception("An arg group is not callable")
-    dummy.__name__ = name
+    __cli_internal__.__name__ = name
 
-    _register_arg_group(name, dummy)
+    _register_arg_group(name, __cli_internal__)
     spec = {
         "kind":   "arg",
         "args":   args,
         "kwargs": kwargs,
         "name":   name
     }
-    dummy.__cmd_info__['arg_info'].append(spec)
+    __cli_internal__.__cmd_info__['arg_info'].append(spec)
 
-    return dummy
+    return __cli_internal__
 
 
 # ------------------------------------------------------------------------------------------------------------------------
@@ -832,7 +834,7 @@ def add_command_subparser_args(
         kind = arg_info['kind']
         if kind == "arg_group":
             name = arg_info['name']
-            if name not in __ARG_GROUPS__:
+            if name not in __ARG_GROUPS__:  # pragma: no cover
                 cmd = subparser.get_default('command_name')
                 gcs = ' -> '.join(_group_chain)
                 gcm = '' if not gcs else f" indirectly via group(s): {gcs}"
@@ -846,9 +848,9 @@ def add_command_subparser_args(
             args   = arg_info['args']
             kwargs = arg_info['kwargs']
             subparser.add_argument(*args, **kwargs)
-        else:
+        else:  # pragma: no cover
             raise Exception(f"Unknown argument spec type: '{kind}'")
-    else:
+    else:  # pragma: no cover
         raise TypeError(f"Invalid type for argument spec: '{type(arg_info).__name__}'")
 
 
@@ -859,21 +861,21 @@ __PARSER__ = None
 
 # ------------------------------------------------------------------------------------------------------------------------
 def get_args_parser(
-    program:           str,
-    version:           str,
-    copyright_message: Optional[str]
+    program:           Optional[str] = None,
+    version:           Optional[str] = None,
+    copyright_message: Optional[str] = None
 ) -> argparse.ArgumentParser:
     """
-    Get ath global argument parser for all registered commands and associated arguments.
+    Get the global argument parser for all registered commands and associated arguments.
 
     Parameters
     ----------
-    program : str
-        The name of the program
+    program : str, optional
+        The name of the program. Defaults to `sys.argv[0]`
     version : str
-        The current version of the program.
+        The current version of the program. Defaults to 'v?.?.?'
     copyright_message : Optional[str]
-        The copyright message for the program.
+        The copyright message for the program. Defaults to '(C) copyright YEAR. All rights reserved)'
 
     Returns
     -------
@@ -883,6 +885,16 @@ def get_args_parser(
     """
     global __PARSER__
     if __PARSER__ is None:
+
+        # set defaults
+        year = datetime.datetime.utcnow().year
+        if copyright_message is None:
+            copyright_message = f"(C) Copyright {year}. All rights reserved."
+        if version is None:
+            version = "v?.?.?"
+        if program is None:
+            program = sys.argv[0]
+
         # initialize the main parser
         parser = argparse.ArgumentParser(
             prog = sys.argv[0],
@@ -960,29 +972,59 @@ def get_args_parser(
 
 
 # ------------------------------------------------------------------------------------------------------------------------
-def run_command() -> int:
+def run_command(
+    args:              Optional[Iterable[str]] = None,
+    program:           Optional[str] = None,
+    version:           Optional[str] = None,
+    copyright_message: Optional[str] = None,
+) -> int:
     """
     Parse supplied command line arguments and run the corresponding command if specified.
 
+    Parameters
+    ----------
+    args: Iterable[str], optional
+        A list of positional arguments excluding the program name, defaults to `sys.argv[1:]`
+    program : str, optional
+        The name of the program. Defaults to `sys.argv[0]`
+    version : str
+        The current version of the program. Defaults to 'v?.?.?'
+    copyright_message : Optional[str]
+        The copyright message for the program. Defaults to '(C) copyright YEAR. All rights reserved)'
+
     Returns
     -------
     int
         The command's exit code.
     """
 
+    year = datetime.datetime.utcnow().year
+    if copyright_message is None:
+        copyright_message = f"(C) Copyright {year}. All rights reserved."
+    if version is None:
+        version = "v?.?.?"
+    if program is None:
+        program = sys.argv[0]
+    if args is None:
+        args = sys.argv[1:]
+
     # get the parser
     try:
-        parser = get_args_parser()
-    except:  # noqa F722
+        parser = get_args_parser(
+            program = program,
+            version = version,
+            copyright_message = copyright_message
+        )
+    except:  # noqa F722  # pragma : no cover
         logger.exception("Error building parser(s).")
         return -1
 
     # parse the args
     try:
-        args = parser.parse_args()
+        args = parser.parse_args( args = args )
     except SystemExit:
         raise
-    except:  # noqa F722
+    except:  # noqa F722  # pragma : no cover
         logger.exception("Error parsing arguments.")
         parser.print_help(file=sys.stderr)
         return -1
@@ -993,13 +1035,19 @@ def run_command() -> int:
             args.__dict__[k] = None
 
     # set log verbosity
-    quiet     = getattr(args, '__quiet', UNDEFINED)
-    if is_defined(quiet):
-        delattr(args, '__quiet')
     verbosity = getattr(args, 'verbosity', UNDEFINED)
-    if is_defined(verbosity):
+    if is_defined(verbosity):  # pragma : no cover
         delattr(args, 'verbosity')
-    set_verbosity( DEFAULT_VERBOSITY + default_if_undefined(verbosity, 0) - default_if_undefined(quiet, 0) )
+    verbosity = default_if_undefined(verbosity, 0)
+    verbosity = verbosity if (verbosity is not None) else 0
+
+    quiet     = getattr(args, '__quiet', UNDEFINED)
+    if is_defined(quiet):  # pragma : no cover
+        delattr(args, '__quiet')
+    quiet = default_if_undefined(quiet, 0)
+    quiet = quiet if (quiet is not None) else 0
+
+    set_verbosity( DEFAULT_VERBOSITY + verbosity - quiet )
 
     # determine if we have a specified command
     subparser = getattr(args, 'subparser', None)
