@@ -7,6 +7,8 @@
 # ------------------------------------------------------------------------------------------------------------------------
 # Import and pathing setup
 # ------------------------------------------------------------------------------------------------------------------------
+import os
+import sys
 import pathlib
 from typing import Any, Callable, Dict, Generator, Iterable, Optional, Pattern, Union
 
@@ -29,19 +31,253 @@ PathList = Iterable[PathLike]
 # ------------------------------------------------------------------------------------------------------------------------
 # Class: Path
 # ------------------------------------------------------------------------------------------------------------------------
-class Path(pathlib.Path):
-    """ A filesystem path. """
+class Path():
+    """ A filesystem path. this is a wrapper class for `pathlib.Path`. """
+    def _native_of(path: PathLike):
+        if isinstance(path, Path):
+            return path._instance
+        elif isinstance(path, pathlib.Path):
+            return path
+        else:
+            return str(path)
 
     def __init__(self, path: PathLike):
-        """
-        Create a new filesystem path object.
+        self._instance = pathlib.Path(Path._native_of(path))
 
-        Parameters
-        ----------
-        path : PathLike
-            The filesystem path. May be a `Path`, a `pathlib.Path`, a string, or any type deriving from `Path`.
-        """
-        super().__init__(path)
+    def __enter__(self):
+        return self._instance.__enter__()
+
+    def __exit__(self, t, v, tb):
+        self._instance.__exit__(t, v, tb)
+
+    @classmethod
+    def cwd(cls):
+        return cls(os.getcwd())
+
+    @classmethod
+    def home(cls):
+        return cls(cls('.')._instance.home())
+
+    def samefile(self, other_path):
+        return self._instance.samefile(Path._native_of(other_path))
+
+    def iterdir(self):
+        for path in self._instance.iterdir():
+            yield Path(path)
+
+    def glob(self, pattern):
+        for path in self._instance.glob(pattern):
+            yield Path(path)
+
+    def rglob(self, pattern):
+        for path in self._instance.rglob(pattern):
+            yield Path(path)
+
+    def absolute(self):
+        return Path(self._instance.absolute())
+
+    def resolve(self, strict=False):
+        return Path(self._instance.resolve(strict=strict))
+
+    def stat(self):
+        return self._instance.stat()
+
+    def owner(self):
+        return self._instance.owner()
+
+    def group(self):
+        return self._instance.group()
+
+    def open(self, *args, **kwargs):
+        return self._instance.open(*args, **kwargs)
+
+    def read_bytes(self):
+        return self._instance.read_bytes()
+
+    def read_text(self, *args, **kwargs):
+        return self._instance.read_text(*args, **kwargs)
+
+    def write_bytes(self, data):
+        return self._instance.write_bytes(data)
+
+    def write_text(self, *args, **kwargs):
+        return self._instance.write_text(*args, **kwargs)
+
+    def touch(self, *args, **kwargs):
+        self._instance.touch(*args, **kwargs)
+
+    def mkdir(self, *args, **kwargs):
+        self._instance.mkdir(*args, **kwargs)
+
+    def chmod(self, *args, **kwargs):
+        self._instance.chmod(*args, **kwargs)
+
+    def lchmod(self, *args, **kwargs):
+        self._instance.lchmod(*args, **kwargs)
+
+    def unlink(self):
+        self._instance.unlink()
+
+    def rmdir(self):
+        self._instance.rmdir()
+
+    def lstat(self):
+        return self._instance.lstat()
+
+    def rename(self, target):
+        self._instance.rename(Path._native_of(target))
+
+    def replace(self, target):
+        if sys.version_info >= (3, 8):  # pragma: no cover
+            return Path(self._instance.replace(Path._native_of(target)))
+        else:
+            self._instance.replace(Path._native_of(target))
+
+    def symlink_to(self, target, target_is_directory=False):
+        self._instance.symlink_to(
+            target              = Path._native_of(target),
+            target_is_directory = target_is_directory
+        )
+
+    # Convenience functions for querying the stat results
+    def exists(self):
+        return self._instance.exists()
+
+    def is_dir(self):
+        return self._instance.is_dir()
+
+    def is_file(self):
+        return self._instance.is_file()
+
+    def is_symlink(self):
+        return self._instance.is_symlink()
+
+    def is_block_device(self):
+        return self._instance.is_block_device()
+
+    def is_char_device(self):
+        return self._instance.is_char_device()
+
+    def is_fifo(self):
+        return self._instance.is_fifo()
+
+    def is_socket(self):
+        return self._instance.is_socket()
+
+    def expanduser(self):
+        return Path(self._instance.expanduser())
+
+    def __str__(self):
+        return str(self._instance)
+
+    def __fspath__(self):
+        return str(self)
+
+    def as_posix(self):
+        return self._instance.as_posix()
+
+    def __bytes__(self):
+        return self._instance.__bytes__()
+
+    def __repr__(self):
+        return f"{type(self).__name__}('{str(self)}')"
+
+    def as_uri(self):
+        return self._instance.as_uri()
+
+    def __eq__(self, other):
+        return self._instance == pathlib.Path(Path._native_of(other))
+
+    def __ne__(self, other):
+        return self._instance != pathlib.Path(Path._native_of(other))
+
+    def __hash__(self):
+        return self._instance.__hash__()
+
+    def __lt__(self, other):
+        return self._instance < pathlib.Path(Path._native_of(other))
+
+    def __le__(self, other):
+        return self._instance <= pathlib.Path(Path._native_of(other))
+
+    def __gt__(self, other):
+        return self._instance > pathlib.Path(Path._native_of(other))
+
+    def __ge__(self, other):
+        return self._instance >= pathlib.Path(Path._native_of(other))
+
+    @property
+    def drive(self):
+        return self._instance.drive
+
+    @property
+    def root(self):
+        return self._instance.root
+
+    @property
+    def anchor(self):
+        return self._instance.anchor
+
+    @property
+    def name(self):
+        return self._instance.name
+
+    @property
+    def suffix(self):
+        return self._instance.suffix
+
+    @property
+    def suffixes(self):
+        return self._instance.suffixes
+
+    @property
+    def stem(self):
+        return self._instance.stem
+
+    def with_name(self, name):
+        return Path(self._instance.with_name(name=name))
+
+    def with_suffix(self, suffix):
+        return Path(self._instance.with_suffix(suffix=suffix))
+
+    def relative_to(self, *other):
+        return Path(
+            self._instance.relative_to(
+                *(
+                    Path._native_of(x) for x in other
+                )
+            )
+        )
+
+    @property
+    def parts(self):
+        return self._instance.parts
+
+    def joinpath(self, *args):
+        return Path(self._instance.joinpath(*args))
+
+    def __truediv__(self, key):
+        return Path(self._instance / key)
+
+    def __rtruediv__(self, key):
+        return Path(key / self._instance)
+
+    @property
+    def parent(self):
+        return Path(self._instance.parent)
+
+    @property
+    def parents(self):
+        return [Path(x) for x in self._instance.parents]
+
+    def is_absolute(self):
+        return self._instance.is_absolute()
+
+    def is_reserved(self):
+        return self._instance.is_reserved()
+
+    def match(self, path_pattern):
+        return self._instance.match(path_pattern)
 
     # --------------------------------------------------------------------------------------------------------------------
     @staticmethod
@@ -98,7 +334,7 @@ class Path(pathlib.Path):
         """
         reference = Path.path_of(reference)
         for path in paths:
-            yield pathlib.Path(path).relative_to(reference)
+            yield Path(path).relative_to(reference)
 
 
 # =======================================================================================================================
@@ -106,6 +342,9 @@ class Path(pathlib.Path):
 # =======================================================================================================================
 class File(Path):
     """ Class to simplify common file operations. """
+    # def __new__(cls, *args, **kwargs):
+    #     klass = ShimWindowsFile if os.name == 'nt' else ShimPosixFile
+    #     return super().__new__(klass, *args, **kwargs)
 
     # --------------------------------------------------------------------------------------------------------------------
     def __init__(self, path: PathLike):
@@ -118,8 +357,8 @@ class File(Path):
             The filesystem path of a file. This file may or may not exist.
         """
         super().__init__(path)
-        self._loader  = None
-        self._saver   = None
+        self._loader = None
+        self._saver = None
 
     # --------------------------------------------------------------------------------------------------------------------
     def exists(self) -> bool:
@@ -369,7 +608,7 @@ class Folder(Path):
         Folder
             Any child folder.
         """
-        return self.search_subdirectories(self)
+        return self.search_subdirectories()
 
     # --------------------------------------------------------------------------------------------------------------------
     @property
@@ -382,7 +621,7 @@ class Folder(Path):
         Folder
             Any child file.
         """
-        return self.search_files(self)
+        return self.search_files()
 
 
 # =======================================================================================================================
@@ -506,4 +745,7 @@ class FolderSet:
                     )
             return File(path=child)
 
-        raise KeyError(f"Object '{name}' found in directory '{self._root}' is neither a file nor a directory.")
+        raise KeyError(  # pragma: no cover
+            f"Object '{name}' found in directory '{self._root}' "
+            "is neither a file nor a directory."
+        )
