@@ -43,7 +43,7 @@ __COPIED_ENV_ARGNAMES__ = [
 # =======================================================================================================================
 # Class: Program
 # =======================================================================================================================
-class Program(object):
+class Program(object):  # pragma: no cover
     """ A child process. """
 
     # --------------------------------------------------------------------------------------------------------------------
@@ -213,6 +213,8 @@ class Program(object):
         cwd : Optional[str], optional
             When specified, change the current directory to the path provided, by default None
         """
+        if is_string(args):
+            args = args.split(' ')
         self.args             = [str(arg) for arg in args]
         self.capture_stdout   = capture_stdout
         self.capture_stderr   = capture_stderr
